/*----------------------------------------------------------------------------
/ SCI}N											2005 (C)LogicPack
/	05.01						000		M.Yamada
/		first release(M-801)
/	05.07.22					001		M.Yamada
/		G[NALq~XC
/---------------------------------------------------------------------------*/
#include "iodefine.h"
#include "sci.h"

/*-----------------------------------------------
/ *DEFINE
/----------------------------------------------*/

/*-----------------------------------------------
/ *STATIC
/----------------------------------------------*/
/* tx buffer */
static byte tx_buffer[TX_BUFSIZE];
static volatile byte *tx_rp, *tx_wp;
/* rx buffer */
static byte rx_buffer[RX_BUFSIZE];
static volatile byte *rx_rp, *rx_wp;
/* err */
static volatile byte sci_err;


/*-------------------------------------------------------------------
/ 
/------------------------------------------------------------------*/
void SCI_INIT(void)
{
	tx_rp = tx_wp = &tx_buffer[0];
	rx_rp = rx_wp = &rx_buffer[0];

#ifdef TX_SIMPLE
	SCIx.SCR.BIT.TIE = 0;
#else
//	SCIx.SCR.BIT.TIE = 1;
	SCIx.SCR.BIT.TIE = 0;
#endif
	SCIx.SCR.BIT.RIE  = 1;
#ifdef TX_SIMPLE
	SCIx.SCR.BIT.TE   = 1;
#else
//	SCIx.SCR.BIT.TE   = 0;
	SCIx.SCR.BIT.TE   = 1;
#endif
	SCIx.SCR.BIT.RE   = 1;
	SCIx.SCR.BIT.MPIE = 0;
	SCIx.SCR.BIT.TEIE = 0;
}
/*-------------------------------------------------------------------
/ PM
/	ENT:
/	  byte data						tx data
/	RET:
/	  int							0:OK
/									!0:NG
/------------------------------------------------------------------*/
int SCI_PUT(byte data)
{
#ifdef TX_SIMPLE				/* simple */
	if(SCIx.SSR.BIT.TDRE == 0){ return 1; }

	SCIx.TDR = data;
	SCIx.SSR.BIT.TDRE = 0;
	return 0;
#else
	bool full;
	byte *wp;

	full = sci_err & TXBUF_FULL;
	if( full ){
		return 1;
	}

	*tx_wp = data;
	wp = tx_wp + 1;
	if(wp == &tx_buffer[TX_BUFSIZE]){
		wp = &tx_buffer[0];
	}
	tx_wp = wp;

	if(tx_wp == tx_rp){
		sci_err |= TXBUF_FULL;
	}
	return 0;

#endif
}
/*-------------------------------------------------------------------
/ MgK
/------------------------------------------------------------------*/
void SCI_TXTRG(void)
{
#ifndef TX_SIMPLE
	if(tx_wp == tx_rp){
		if( !(sci_err & TXBUF_FULL) ){ return; }
	}
//	SCIx.SCR.BIT.TE = 1;		/* MJn */
	SCIx.SCR.BIT.TIE = 1;		/* MJn */
#endif
}

/*-------------------------------------------------------------------
/ PM
/	ENT:
/	  int ch						SCI no
/	RET:
/	  int							& 0x100 -> f[^L
/									ʃoCg -> Mf[^
/------------------------------------------------------------------*/
int SCI_GET(void)
{
	bool full;
	int data;
	byte *rp;

	full = sci_err & RXBUF_FULL;

	if( !full ){
		if(rx_rp == rx_wp){ return 0; }
	}

	data = *rx_rp | 0x100;
	rp = rx_rp + 1;
	if(rp == &rx_buffer[RX_BUFSIZE]){
		rp = &rx_buffer[0];
	}
	rx_rp = rp;

	if( full ){
		sci_err &= ~RXBUF_FULL;
		SCIx.SCR.BIT.RE   = 1;		/* MJn */
	}
	return data;
}
/*-------------------------------------------------------------------
/ M
/	ENT:
/	  byte *buf						Mf[^
/	  int len						Mf[^
/	RET:
/	  int							Mf[^
/------------------------------------------------------------------*/
int SCI_PUTS(const byte *buf, int len)
{
	int max = 0;
	while(len > 0){
		if(SCI_PUT(*buf) != 0){ break; }
		max++;
		len--;
		buf++;
	}
	if(max > 0){ SCI_TXTRG(); }
	return max;
}
/*-------------------------------------------------------------------
/ M
/	ENT:
/	  byte *buf						Mf[^obt@
/	  int max						obt@ő咷
/	RET:
/	  int							Mf[^
/------------------------------------------------------------------*/
int SCI_GETS(byte *buf, int max)
{
	int len = 0;
	int data;
	while(max > 0){
		data = SCI_GET();
		if( !(data & 0x100) ){ break; }		/* f[^ */
		*buf = (byte)data;
		max--;
		len++;
		buf++;
	}
	return len;
}

/*-------------------------------------------------------------------
/ Mobt@󂫃TCY̎擾
/------------------------------------------------------------------*/
int SCI_GETTXFREE(void)
{
	int len;

	if(tx_wp >= tx_rp){
		len = (tx_wp - tx_rp) + TX_BUFSIZE;
	}else{
		len = tx_rp - tx_wp;
	}
	return 0;
}

/*-------------------------------------------------------------------
/ Mobt@󂫃TCY̎擾
/------------------------------------------------------------------*/
int SCI_GETRXFREE(void)
{
	int len;

	if(rx_wp >= rx_rp){
		len = (rx_wp - rx_rp) + RX_BUFSIZE;
	}else{
		len = rx_rp - rx_wp;
	}
	return 0;
}

/*-------------------------------------------------------------------
/ G[̎擾
/------------------------------------------------------------------*/
byte SCI_GETERR(void)
{
	return sci_err;
}

#pragma section IntPRG
/*-------------------------------------------------------------------
/ MG[荞
/------------------------------------------------------------------*/
INT_ERI(void)
{
#if 0
	if(SCIx.SSR.BIT.FER == 1){ SCIx.SSR.BIT.FER == 0; }
	if(SCIx.SSR.BIT.PER == 1){ SCIx.SSR.BIT.PER == 0; }
	if(SCIx.SSR.BIT.ORER == 1){ SCIx.SSR.BIT.ORER == 0; }
#else															/*MOD 001*/
	if(SCIx.SSR.BIT.FER == 1){ SCIx.SSR.BIT.FER = 0; }
	if(SCIx.SSR.BIT.PER == 1){ SCIx.SSR.BIT.PER = 0; }
	if(SCIx.SSR.BIT.ORER == 1){ SCIx.SSR.BIT.ORER = 0; }
#endif
}
/*-------------------------------------------------------------------
/ M荞
/------------------------------------------------------------------*/
INT_RXI(void)
{
	if(SCIx.SSR.BIT.RDRF == 1){
		*rx_wp = SCIx.RDR;
		if(++rx_wp == &rx_buffer[RX_BUFSIZE]){
			rx_wp = &rx_buffer[0];
			if(rx_wp == rx_rp){
				sci_err |= RXBUF_FULL;
				SCIx.SCR.BIT.RE   = 0;		/* M~ */
			}
		}
		SCIx.SSR.BIT.RDRF = 0;
	}
}
/*-------------------------------------------------------------------
/ M荞
/------------------------------------------------------------------*/
INT_TXI(void)
{
	if(SCIx.SSR.BIT.TDRE == 1){
		SCIx.TDR = *tx_rp;
		if(++tx_rp == &tx_buffer[TX_BUFSIZE]){
			tx_rp = &tx_buffer[0];
		}
		if(tx_rp == tx_wp){		/* empty */
//			SCIx.SCR.BIT.TE   = 0;		/* M~ */
			SCIx.SCR.BIT.TIE   = 0;		/* M~ */
		}
		sci_err &= ~TXBUF_FULL;
		SCIx.SSR.BIT.TDRE = 0;
	}
}
